<?php

class NGG_NivoSlider extends WP_Widget {

	function NGG_NivoSlider() {
		$widget_ops = array('classname' => 'ngg-nivoslider', 'description' => "Allows you to pick a gallery from the 'NextGen Gallery' plugin to use as a Nivo Slider.");
		$this->WP_Widget('nextgen-nivoslider', 'NextGEN NivoSlider', $widget_ops);
	}

	function widget($args, $instance) {
		global $wpdb;
		extract($args);

		// Set params
		$title = apply_filters('widget_title', (isset($instance['title']) ? $instance['title'] : ""));
		$htmlID = $this->get_val($instance, 'html_id', 'slider');
		$width = $this->get_val_numeric($instance, 'width');
		$height = $this->get_val_numeric($instance, 'height');
		$resizeWidth = $this->get_val_numeric($instance, 'resizewidth');
		$resizeHeight = $this->get_val_numeric($instance, 'resizeheight');
		$resizeBGColor = $this->get_val($instance, 'resizebg');
		$resize = $this->get_val($instance, 'resize');
		$order = $this->get_val($instance, 'order', 'asc', false);
		$tags = $this->get_val($instance, 'tags');
		$shuffle = $this->get_val($instance, 'shuffle');
		$limit = $this->get_val_numeric($instance, 'max_pictures');
		$center = $this->get_val($instance, 'center');
		$gallery = $this->get_val_numeric($instance, 'gallery');
		$shortCode = $this->get_val($instance, 'shortcode');
		$caption = $this->get_val($instance, 'caption');

		// Set nivo params
		$effect = $this->get_val($instance, 'effect');
		$slices = $this->get_val($instance, 'slices');
		$boxCols = $this->get_val($instance, 'boxcols');
		$boxRows = $this->get_val($instance, 'boxrows');
		$animSpeed = $this->get_val($instance, 'animspeed');
		$pauseTime = $this->get_val($instance, 'pausetime');
		$startSlide = $this->get_val($instance, 'startslide');
		$directionNav = $this->get_val($instance, 'directionnav');
		$controlNav = $this->get_val($instance, 'controlnav');
		$controlNavThumbs = $this->get_val($instance, 'controlnavthumbs');
		$pauseOnHover = $this->get_val($instance, 'pauseonhover');
		$thumbsWidth = $this->get_val_numeric($instance, 'thumbswidth');
		$thumbsHeight = $this->get_val_numeric($instance, 'thumbsheight');
		$thumbsContainerHeight = $this->get_val_numeric($instance, 'thumbscontainerheight');
		$thumbsGap = $this->get_val_numeric($instance, 'thumbsgap');
		$keyboardNav = $this->get_val($instance, 'keyboardnav');
		$manualAdvance = $this->get_val($instance, 'manualadvance');
		$prevText = $this->get_val($instance, 'prevtext');
		$nextText = $this->get_val($instance, 'nexttext');
		$randomStart = $this->get_val($instance, "randomstart");
		$captionOpacity = $this->get_val($instance, 'captionopacity');
		$disableCaptions = $this->get_val($instance, 'disablecaptions', 1);
		$beforeChange = $this->get_val($instance, 'beforechange', '', false);
		$afterChange = $this->get_val($instance, 'afterchange', '', false);
		$slideshowEnd = $this->get_val($instance, 'slideshowend', '', false);
		$lastSlide = $this->get_val($instance, 'lastslide', '', false);
		$afterLoad = $this->get_val($instance, 'afterload', '', false);

		// SQL defaults
		$sqlOrder = '';
		$sqlWhere = ' WHERE exclude = 0';
		$sqlLimit = '';

		// Set SQL order
		switch ($order) {
			case 'random':
				$sqlOrder = 'RAND()';
				break;
			case 'sortorder':
				$sqlOrder = 'sortorder ASC';
				break;
			case 'desc':
				$sqlOrder = 'galleryid DESC';
				break;
			case 'asc':
			default:
				$sqlOrder = 'galleryid ASC';
		}

		if($gallery != '') {
			$sqlWhere .= ' AND galleryid = ' . $gallery;
		}

		// Set limit defaults only if tags are not being used
		$numLimit = -1;
		if(is_numeric($limit)) {
			$numLimit = (int)$limit;
			if($tags == '') {
				$sqlLimit = ' LIMIT 0, ' . $limit;
			}
		}

		$results = $wpdb->get_results("SELECT * FROM $wpdb->nggpictures" . $sqlWhere . " ORDER BY " . $sqlOrder . $sqlLimit);
		$pSize = 0;
		if(is_array($results)) {
			// Filter by tag if entered
			if($tags != '') {
				$taggedImages = nggTags::find_images_for_tags($tags);

				if($taggedImages) {
					$taggedImageIDs = array();
					foreach($taggedImages as $image){
						$taggedImageIDs[] = $image->pid;
					}

					if(sizeof($taggedImageIDs) > 0) {

						$filteredResults = array();
						$taggedCount = 0;
						foreach($results as $result) {
							if($numLimit != -1 && $taggedCount >= $numLimit) {
								break;
							} else {
								if(in_array($result->pid, $taggedImageIDs)) {
									$filteredResults[] = $result;
										$taggedCount += 1;
								}
							}
						}

						$results = $filteredResults;
					}

				} else {
					$results = array();
				}
			}

			$pSize = count($results);
		}

		$output = '';
		if($pSize > 0) {
			if($title != '') {
				if($shortCode != '1') {
					$output .= "\n" . $before_title . $title . $after_title;
				} else {
					$output .= "\n<h3>" . $title . "</h3>";
				}
			}

			$hasControlNav = ($controlNav == '' || $controlNav == 'true');
			$hasThumbs = ($controlNavThumbs == 'true' || $controlNavThumbs == 'nextgen_thumbs' || $controlNavThumbs == 'nextgen_original');
			$hasCenter = $center == '1' && $width != '';

			if($width != '' || $height != '') {
				$width_s = '';
				$height_s = '';

				if($width != '') {
					$width_s = "width: " . $width . "px !important;";
				}

				if($height != '') {
					$height_s = "height: " . $height . "px !important;";
				}
				$output .= "\n<style type=\"text/css\">";

				if($width_s != '' || $height_s != '') {
					$output .= "\n  div#" . $htmlID . " { " . $width_s . $height_s . " }";
				}

				if($width_s != '') {
					$output .= "\n  div#" . $htmlID . "_container .nivo_slider .nivo-controlNav { " . $width_s . " }";
				}

				if($hasControlNav && $hasThumbs) {
					if($thumbsContainerHeight != '') {
						$output .= "\n  div#" . $htmlID . "_container { padding-bottom: " . $thumbsContainerHeight . "px; }";
						$output .= "\n  div#" . $htmlID . "_container .nivo-controlNav { bottom: -" . $thumbsContainerHeight . "px; }";
					}

					if ($thumbsGap != '' || $thumbsWidth != '' || $thumbsHeight != '') {
						$output .= "\n  div#" . $htmlID . "_container .nivo-controlNav img { ";
						if($thumbsGap != '') {
							$output .= "margin-right: " . $thumbsGap . "px;";
						}

						if($thumbsWidth != '') {
							$output .= "width: " . $thumbsWidth . "px ;";
						}

						if($thumbsHeight != '') {
							$output .= "height: " . $thumbsHeight . "px;";
						}
						$output .= " }";
					}
					if($hasCenter && $thumbsGap != '') {
						$output .= "\n  div#" . $htmlID . "_container .nivo-controlNav img.first_thumb { margin-left: " . $thumbsGap . "px; }";
					}
				}
				$output .= "\n</style>";
			}

			$containerClass = '';
			if($hasCenter) {
				$containerClass = ' nivo_slider_center';
			}
			if($hasControlNav) {
				if($hasThumbs) {
					$containerClass .= ' nivo_slider_controlNavImages';
				} else {
					$containerClass .= ' nivo_slider_controlNavText';
				}
			}

			$output .= "\n<div id=\"" . $htmlID . "_container\" class=\"nivo_slider_container" . $containerClass . "\">";
			$output .= "\n  <div id=\"" . $htmlID . "\" class=\"nivo_slider\">";
			$imageAltText = null;
			$imageDescription = null;
			foreach($results as $result) {
				$gallery = $wpdb->get_row("SELECT * FROM $wpdb->nggallery WHERE gid = '" . $result->galleryid . "'");
				foreach($gallery as $key => $value) {
					$result->$key = $value;
				}

				$image = new nggImage($result);
				$imageAltText = trim($image->alttext);
				$imageDescription = trim($image->description);

				$output .= "\n    ";

				// Check if alt is URL
				$use_url = false;

				if($imageAltText != '' && preg_match('%\A/|\Ahttp://|\Aftp://%', $imageAltText)) {
					$use_url = true;
					$output .= "<a href=\"" . esc_attr($imageAltText) . "\">";
				}

				// Add a caption containing the image alttext and/or description
				if ($caption != '') {
					$disableCaptions = 0;
					$imageTitle = null;

					if ($caption == 'both') {
						$imageTitle = 'title="<b>' . $imageAltText . '</b> - ' . $imageDescription . '"';
					} elseif ($caption == 'alttext') {
						$imageTitle = 'title="' . $imageAltText . '"';
					}elseif ($caption == 'description') {
						$imageTitle = 'title="' . $imageDescription . '"';
					}
				}

				//$output .= "<img src=\"" . $image->imageURL . "\" " . $imageTitle . "class=\"nivo_image\" alt=\"nivo slider image\" />";
				$output .= "<img src=\"";

				// Resizing functions here
				// TimThumb takes option 0-3
				if($resize != '' && $resize <= 4) {
					$output .= plugins_url('includes/timthumb.php' , dirname(__FILE__)) . '?zc=' . ($resize - 1) . '&cc=' . $resizeBGColor . '&src=' . $image->path . '/' . $image->filename;
					if ($resizeHeight != '' || $resizeWidth != '') {
						if ($resizeHeight) {
							$output .= '&h=' . $resizeHeight;
						}
						if ($resizeWidth) {
							$output .= '&w=' . $resizeWidth;
						}

					} else {
						if ($height) {
							$output .= '&h=' . $height;
						}
						if ($width) {
							$output .= '&w=' . $width;
						}
					}

				} else {
					$output .= $image->imageURL;
				}

				$output .= '" ' . $imageTitle ; //. ' border="0" class="nivo_image" alt="nivo slider image"';

				// Add data-thumb attribute for thumbnail navigation as per NivoSlider v3.0+
				if ($hasThumbs) {
					$output .= ' data-thumb="' . $image->thumbURL . '" />';
				} else {
					$output .= " />";
				}

				if($use_url != '') {
					$output .= "</a>";
				}
			}
			$output .= "\n  </div>";
			$output .= "\n</div>";
		}

		// Nivo arguments
		$javascriptArgs = array();

		// Modifications if only 1 picture
		if($pSize <= 1) {
			$startSlide = '0';
			$directionNav = 'false';
			$controlNav = 'false';
			$keyboardNav = 'false';
			$pauseOnHover = 'false';
			$manualAdvance = 'false';
			$beforeChange = '';
			$afterChange = '';
			$slideshowEnd = '';
			$lastSlide = '';
			$afterLoad  = '';
		}

		if($effect != "") { $javascriptArgs[] = "effect: '" . $effect . "'"; }
		if($slices != "") { $javascriptArgs[] = "slices: " . $slices . ""; }
		if($boxCols != "") { $javascriptArgs[] = "boxCols: " . $boxCols . ""; }
		if($boxRows != "") { $javascriptArgs[] = "boxRows: " . $boxRows . ""; }
		if($animSpeed != "") { $javascriptArgs[] = "animSpeed: " . $animSpeed; }
		if($pauseTime != "") { $javascriptArgs[] = "pauseTime: " . $pauseTime; }
		if($startSlide != "") { $javascriptArgs[] = "startSlide: " . $startSlide; }
		if($directionNav != "") { $javascriptArgs[] = "directionNav: " . $directionNav; }
		if($controlNav != "") { $javascriptArgs[] = "controlNav: " . $controlNav; }
		if($controlNavThumbs != "") { $javascriptArgs[] = "controlNavThumbs: " . ($hasThumbs ? 'true' : 'false'); }
		if($keyboardNav != "") { $javascriptArgs[] = "keyboardNav: " . $keyboardNav; }
		if($pauseOnHover != "") { $javascriptArgs[] = "pauseOnHover: " . $pauseOnHover; }
		if($manualAdvance != "") { $javascriptArgs[] = "manualAdvance: " . $manualAdvance; }
		if($prevText != "") { $javascriptArgs[] = "prevText: " . $prevText; }
		if($nextText != "") { $javascriptArgs[] = "nextText: " . $nextText; }
		if($randomStart != "") { $javascriptArgs[] = "randomStart: " . $randomStart; }
		if($captionOpacity != "") { $javascriptArgs[] = "captionOpacity: " . $captionOpacity; }
		if($beforeChange != "") { $javascriptArgs[] = "beforeChange: " . $beforeChange; }
		if($afterChange != "") { $javascriptArgs[] = "afterChange: " . $afterChange; }
		if($slideshowEnd != "") { $javascriptArgs[] = "slideshowEnd: " . $slideshowEnd; }
		if($lastSlide != "") { $javascriptArgs[] = "lastSlide: " . $lastSlide; }
		if($afterLoad != "") { $javascriptArgs[] = "afterLoad: " . $afterLoad; }

		// Add javascript
		$output .= "\n<script type=\"text/javascript\">";
		$output .= "\n  jQuery(window).load(function() {";

		// Shuffle results on random order so even if page is cached the order will be different each time
		if($order == 'random' && $shuffle == 'true') {
			$output .= "\n    jQuery('div#" . $htmlID . "').jj_ngg_shuffle();";
		}
		$output .= "\n    jQuery('div#" . $htmlID . "').nivoSlider(";

		if(count($javascriptArgs) > 0) {
			$output .= "{" . implode(",", $javascriptArgs) . "}";
		}
		$output .= ");";
		if($hasControlNav && $hasThumbs) {
			if($controlNavThumbs == 'nextgen_thumbs' || $controlNavThumbs == 'nextgen_original') {
				$output .= "\n    JJNGGUtils.wordpressThumbs('" . $htmlID . "', " . ($controlNavThumbs == 'nextgen_thumbs' ? 'true' : 'false') . ");";
			}

			if($hasCenter && $thumbsGap != '') {
				$output .= "\n    JJNGGUtils.wordpressThumbsCenterFix('" . $htmlID . "');";
			}

			// Make thumbnails visisble
			$output .= "\n    jQuery('div.nivo-controlNav').css('visibility', 'visible');";
		}
		$output .= "\n  });";
		$output .= "\n</script>\n";

		if($shortCode != '1') {
			echo $before_widget . "\n<ul class=\"ul_jj_slider\">\n    <li class=\"li_jj_slider\">" . $output . "\n    </li>\n  </ul>\n" . $after_widget;
		} else {
			echo $output;
		}
	}

	function get_val($instance, $key, $default = '', $escape = true) {
		$val = '';

		if(isset($instance[$key])) {
			$val = trim($instance[$key]);
		}

		if($val == '') {
			$val = $default;
		}

		if($escape) {
			$val = esc_attr($val);
		}

		return $val;
	}

	function get_val_numeric($instance, $key, $default = '') {
		$val = $this->get_val($instance, $key, $default, false);

		if($val != '' && !is_numeric($val)) {
			$val = '';
		}

		return $val;
	}

	function quote_fix($phrase) {
		return str_replace(array('"', '\"', "\'"), array("'", "'", "'"), $phrase);
	}

	function form($instance) {
		global $wpdb;
		$instance = wp_parse_args((array) $instance, array(
		'title' => '',
		'gallery' => '',
		'html_id' => 'slider',
		'width' => '',
		'height' => '',
		'resizewidth' => '',
		'resizeheight' => '',
		'resizebg' => '',
		'resize' => '',
		'order' => 'random',
		'tags' => '',
		'shuffle' => 'false',
		'max_pictures' => '',
		'center' => '',

		// nivo settings
		'effect' => '',
		'slices' => '',
		'boxcols' => '',
		'boxrows' => '',
		'animspeed' => '',
		'pausetime' => '',
		'startslide' => '',
		'directionnav' => '',
		// 'directionnavhide' => '',
		'controlnav' => '',
		'controlnavthumbs' => '',
		'thumbswidth' => '',
		'thumbsheight' => '',
		'thumbscontainerheight' => '',
		'thumbsgap' => '',
		'controlnavthumbsfromrel' => '',
		'controlnavthumbssearch' => '',
		'controlnavthumbsreplace' => '',
		'keyboardnav' => '',
		'pauseonhover' => '',
		'manualadvance' => '',
		'captionopacity' => '',
		'beforechange' => '',
		'afterchange' => '',
		'slideshowend' => '',
		'lastslide' => '',
		'afterload' => ''
		));

		$order_values = array('random' => 'Random', 'asc' => 'Latest First', 'desc' => 'Oldest First', 'sortorder' => 'NextGen Sortorder');
		$galleries = $wpdb->get_results("SELECT * FROM $wpdb->nggallery ORDER BY name ASC");

?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><strong>Widget title:</strong></label><br />
			<input type="text" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" value="<?php echo $instance['title']; ?>"  class="widefat" />
		</p>
		<p>
			<label><strong>Select a gallery to use:</strong></label><br />

			<?php
			if(is_array($galleries) && count($galleries) > 0) { ?>

				<select id="<?php echo $this->get_field_id('gallery'); ?>" name="<?php echo $this->get_field_name('gallery'); ?>" class="widefat">
					<option value="">All images</option>
					<?php
						$gallery_selected = '';
						foreach($galleries as $gallery) {
							if($gallery->gid == $instance['gallery']) {
								$gallery_selected = " selected=\"selected\"";
							} else {
								$gallery_selected = "";
							}
							echo "<option value=\"" . $gallery->gid . "\"" . $gallery_selected . ">" . $gallery->name . "</option>";
						} ?>
				</select>

			<?php
			} else { ?>No galleries found<?php
			} ?>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('order'); ?>"><strong>Order:</strong></label><br />
			<select id="<?php echo $this->get_field_id('order'); ?>" name="<?php echo $this->get_field_name('order'); ?>" class="widefat">
				<?php
				$order_selected = '';
				foreach($order_values as $key => $value) {
					if($key == $instance['order']) {
						$order_selected = " selected=\"selected\"";
					} else {
						$order_selected = "";
					}
					echo "<option value=\"" . $key . "\"" . $order_selected . ">" . $value . "</option>";
				} ?>
			</select>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('tags'); ?>"><strong>Only display images tagged:</strong></label><br /><small>comma separated list</small><br />
			<input type="text" id="<?php echo $this->get_field_id('tags'); ?>" name="<?php echo $this->get_field_name('tags'); ?>" value="<?php echo $instance['tags']; ?>" class="widefat" />
		</p>
		<p>
			<label><strong>Shuffle:</strong> <small>(Only for random order)</small></label><br />
			<input type="radio" id="<?php echo $this->get_field_id('shuffle'); ?>_true" name="<?php echo $this->get_field_name('shuffle'); ?>" value="true" style="vertical-align: middle;"<?php if($instance['shuffle'] == 'true') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('shuffle'); ?>_true" style="vertical-align: middle;">true</label>
			<input type="radio" id="<?php echo $this->get_field_id('shuffle'); ?>_false" name="<?php echo $this->get_field_name('shuffle'); ?>" value="false" style="vertical-align: middle;"<?php if($instance['shuffle'] == 'false') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('shuffle'); ?>_false" style="vertical-align: middle;">false</label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('max_pictures'); ?>"><strong>Max pictures:</strong> <small>(Leave blank for all)</small></label><br />
			<input type="text" id="<?php echo $this->get_field_id('max_pictures'); ?>" name="<?php echo $this->get_field_name('max_pictures'); ?>" value="<?php echo $instance['max_pictures']; ?>" size="3" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('html_id'); ?>"><strong>HTML id:</strong></label><br />
			<input type="text" id="<?php echo $this->get_field_id('html_id'); ?>" name="<?php echo $this->get_field_name('html_id'); ?>" value="<?php echo $instance['html_id']; ?>" class="widefat" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('width'); ?>"><strong>Width:</strong>  <small>(Leave blank for auto)</small></label><br />
			<input type="text" id="<?php echo $this->get_field_id('width'); ?>" name="<?php echo $this->get_field_name('width'); ?>" value="<?php echo $instance['width']; ?>" size="3" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('height'); ?>"><strong>Height:</strong> <small>(Leave blank for auto)</small></label><br />
			<input type="text" id="<?php echo $this->get_field_id('height'); ?>" name="<?php echo $this->get_field_name('height'); ?>" value="<?php echo $instance['height']; ?>" size="3" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('caption'); ?>"><strong>Show caption:</strong></label><br />
			<select id="<?php echo $this->get_field_id('caption'); ?>" name="<?php echo $this->get_field_name('caption'); ?>" class="widefat">
				<option value="0">Disabled</option>
				<option value="alttext">Alt text</option>
				<option value="description">Description</option>
				<option value="both">Both (Alt text - Description)</option>
			</select>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('resize'); ?>"><strong>Resize images:</strong></label><br />
			<select id="<?php echo $this->get_field_id('resize'); ?>" name="<?php echo $this->get_field_name('resize'); ?>" class="widefat">
				<option value="0">Disabled</option>
				<option value="1">Resize to fit (no cropping)</option>
				<option value="2">Crop and resize to best fit the dimensions</option>
				<option value="3">Resize proportionally and add borders</option>
				<option value="4">Resize proportionally with no gaps</option>
			</select>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('resizebg'); ?>"><strong>Resize background color:</strong>  <small>(Format: #rrggbb - Leave blank for white)</small></label><br />
			<input type="text" id="<?php echo $this->get_field_id('resizebg'); ?>" name="<?php echo $this->get_field_name('resizebg'); ?>" value="<?php echo $instance['resizebg']; ?>" size="3" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('resizewidth'); ?>"><strong>Resize Width:</strong>  <small>(Leave blank for auto)</small></label><br />
			<input type="text" id="<?php echo $this->get_field_id('resizewidth'); ?>" name="<?php echo $this->get_field_name('resizewidth'); ?>" value="<?php echo $instance['resizewidth']; ?>" size="3" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('resizeheight'); ?>"><strong>Resize Height:</strong> <small>(Leave blank for auto)</small></label><br />
			<input type="text" id="<?php echo $this->get_field_id('resizeheight'); ?>" name="<?php echo $this->get_field_name('resizeheight'); ?>" value="<?php echo $instance['resizeheight']; ?>" size="3" />
		</p>
		<p>
			<input type="checkbox" id="<?php echo $this->get_field_id('center'); ?>" style="vertical-align: middle;" name="<?php echo $this->get_field_name('center'); ?>" value="1"<?php if($instance['center'] == '1') { echo " checked=\"checked\""; } ?> />
			<label for="<?php echo $this->get_field_id('center'); ?>" style="vertical-align: middle;"><strong>Center content</strong></label><br />
		</p>
		<div class="javascript_settings" style="display: none; border: 1px solid #cccccc; background-color: #f0f0f0;">
			<div style="padding: 10px;">
				<p><a href="http://nivo.dev7studios.com/#usage" target="jj_nextgen_jquery">Visit Nivo configuration page</a></p>
				<p>Leave blank to use defaults.</p>
				<p>
					<label for="<?php echo $this->get_field_id('effect'); ?>"><strong>effect:</strong> <small>(Leave blank for all)</small></label><br />
					<input type="text" id="<?php echo $this->get_field_id('effect'); ?>" name="<?php echo $this->get_field_name('effect'); ?>" value="<?php echo $instance['effect']; ?>" class="widefat" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('slices'); ?>"><strong>slices:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('slices'); ?>" name="<?php echo $this->get_field_name('slices'); ?>" value="<?php echo $instance['slices']; ?>" size="3" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('boxcols'); ?>"><strong>boxCols:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('boxcols'); ?>" name="<?php echo $this->get_field_name('boxcols'); ?>" value="<?php echo $instance['boxcols']; ?>" size="3" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('boxrows'); ?>"><strong>boxRows:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('boxrows'); ?>" name="<?php echo $this->get_field_name('boxrows'); ?>" value="<?php echo $instance['boxrows']; ?>" size="3" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('animspeed'); ?>"><strong>animSpeed:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('animspeed'); ?>" name="<?php echo $this->get_field_name('animspeed'); ?>" value="<?php echo $instance['animspeed']; ?>" size="3" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('pausetime'); ?>"><strong>pauseTime:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('pausetime'); ?>" name="<?php echo $this->get_field_name('pausetime'); ?>" value="<?php echo $instance['pausetime']; ?>" size="3" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('startslide'); ?>"><strong>startSlide:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('startslide'); ?>" name="<?php echo $this->get_field_name('startslide'); ?>" value="<?php echo $instance['startslide']; ?>" size="3" />
				</p>
				<p>
					<label><strong>directionNav:</strong></label><br />
					<input type="radio" id="<?php echo $this->get_field_id('directionnav'); ?>_default" name="<?php echo $this->get_field_name('directionnav'); ?>" value="" style="vertical-align: middle;"<?php if($instance['directionnav'] == '') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('directionnav'); ?>_default" style="vertical-align: middle;">default</label>
					<input type="radio" id="<?php echo $this->get_field_id('directionnav'); ?>_true" name="<?php echo $this->get_field_name('directionnav'); ?>" value="true" style="vertical-align: middle;"<?php if($instance['directionnav'] == 'true') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('directionnav'); ?>_true" style="vertical-align: middle;">true</label>
					<input type="radio" id="<?php echo $this->get_field_id('directionnav'); ?>_false" name="<?php echo $this->get_field_name('directionnav'); ?>" value="false" style="vertical-align: middle;"<?php if($instance['directionnav'] == 'false') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('directionnav'); ?>_false" style="vertical-align: middle;">false</label>
				</p>
				<p>
					<label><strong>controlNav:</strong></label><br />
					<input type="radio" id="<?php echo $this->get_field_id('controlnav'); ?>_default" name="<?php echo $this->get_field_name('controlnav'); ?>" value="" style="vertical-align: middle;"<?php if($instance['controlnav'] == '') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnav'); ?>_default" style="vertical-align: middle;">default</label>
					<input type="radio" id="<?php echo $this->get_field_id('controlnav'); ?>_true" name="<?php echo $this->get_field_name('controlnav'); ?>" value="true" style="vertical-align: middle;"<?php if($instance['controlnav'] == 'true') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnav'); ?>_true" style="vertical-align: middle;">true</label>
					<input type="radio" id="<?php echo $this->get_field_id('controlnav'); ?>_false" name="<?php echo $this->get_field_name('controlnav'); ?>" value="false" style="vertical-align: middle;"<?php if($instance['controlnav'] == 'false') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnav'); ?>_false" style="vertical-align: middle;">false</label>
				</p>
				<p>
					<label><strong>controlNavThumbs:</strong></label><br />
					<input type="radio" id="<?php echo $this->get_field_id('controlnavthumbs'); ?>_default" name="<?php echo $this->get_field_name('controlnavthumbs'); ?>" value="" style="vertical-align: middle;"<?php if($instance['controlnavthumbs'] == '') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnavthumbs'); ?>_default" style="vertical-align: middle;">default</label>
					<input type="radio" id="<?php echo $this->get_field_id('controlnavthumbs'); ?>_true" name="<?php echo $this->get_field_name('controlnavthumbs'); ?>" value="true" style="vertical-align: middle;"<?php if($instance['controlnavthumbs'] == 'true') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnavthumbs'); ?>_true" style="vertical-align: middle;">true</label>
					<input type="radio" id="<?php echo $this->get_field_id('controlnavthumbs'); ?>_false" name="<?php echo $this->get_field_name('controlnavthumbs'); ?>" value="false" style="vertical-align: middle;"<?php if($instance['controlnavthumbs'] == 'false') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnavthumbs'); ?>_false" style="vertical-align: middle;">false</label>
					<br /><input type="radio" id="<?php echo $this->get_field_id('controlnavthumbs'); ?>_nextgen_thumbs" name="<?php echo $this->get_field_name('controlnavthumbs'); ?>" value="nextgen_thumbs" style="vertical-align: middle;"<?php if($instance['controlnavthumbs'] == 'nextgen_thumbs') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnavthumbs'); ?>_nextgen_thumbs" style="vertical-align: middle;">nextgen thumbs</label>
					<br /><small>(auto link to NextGen thumbs)</small>
					<br /><input type="radio" id="<?php echo $this->get_field_id('controlnavthumbs'); ?>_nextgen_original" name="<?php echo $this->get_field_name('controlnavthumbs'); ?>" value="nextgen_original" style="vertical-align: middle;"<?php if($instance['controlnavthumbs'] == 'nextgen_original') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnavthumbs'); ?>_nextgen_original" style="vertical-align: middle;">nextgen original</label>
					<br /><small>(auto link to original image)</small>
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('thumbswidth'); ?>"><strong>thumbsWidth:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('thumbswidth'); ?>" name="<?php echo $this->get_field_name('thumbswidth'); ?>" value="<?php echo $instance['thumbswidth']; ?>" size="3" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('thumbsheight'); ?>"><strong>thumbsHeight:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('thumbsheight'); ?>" name="<?php echo $this->get_field_name('thumbsheight'); ?>" value="<?php echo $instance['thumbsheight']; ?>" size="3" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('thumbscontainerheight'); ?>"><strong>thumbsContainerHeight:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('thumbscontainerheight'); ?>" name="<?php echo $this->get_field_name('thumbscontainerheight'); ?>" value="<?php echo $instance['thumbscontainerheight']; ?>" size="3" />
					<br /><small>(eg, image rows x thumb height)</small>
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('thumbsgap'); ?>"><strong>thumbsGap:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('thumbsgap'); ?>" name="<?php echo $this->get_field_name('thumbsgap'); ?>" value="<?php echo $instance['thumbsgap']; ?>" size="3" />
				</p>
				<p>
					<label><strong>controlNavThumbsFromRel:</strong></label><br />
					<input type="radio" id="<?php echo $this->get_field_id('controlnavthumbsfromrel'); ?>_default" name="<?php echo $this->get_field_name('controlnavthumbsfromrel'); ?>" value="" style="vertical-align: middle;"<?php if($instance['controlnavthumbsfromrel'] == '') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnavthumbsfromrel'); ?>_default" style="vertical-align: middle;">default</label>
					<input type="radio" id="<?php echo $this->get_field_id('controlnavthumbsfromrel'); ?>_true" name="<?php echo $this->get_field_name('controlnavthumbsfromrel'); ?>" value="true" style="vertical-align: middle;"<?php if($instance['controlnavthumbsfromrel'] == 'true') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnavthumbsfromrel'); ?>_true" style="vertical-align: middle;">true</label>
					<input type="radio" id="<?php echo $this->get_field_id('controlnavthumbsfromrel'); ?>_false" name="<?php echo $this->get_field_name('controlnavthumbsfromrel'); ?>" value="false" style="vertical-align: middle;"<?php if($instance['controlnavthumbsfromrel'] == 'false') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('controlnavthumbsfromrel'); ?>_false" style="vertical-align: middle;">false</label>
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('controlnavthumbssearch'); ?>"><strong>controlNavThumbsSearch:</strong></label><br />
					<input type="text" id="<?php echo $this->get_field_id('controlnavthumbssearch'); ?>" name="<?php echo $this->get_field_name('controlnavthumbssearch'); ?>" value="<?php echo $instance['controlnavthumbssearch']; ?>" class="widefat" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('controlnavthumbsreplace'); ?>"><strong>controlNavThumbsReplace:</strong></label><br />
					<input type="text" id="<?php echo $this->get_field_id('controlnavthumbsreplace'); ?>" name="<?php echo $this->get_field_name('controlnavthumbsreplace'); ?>" value="<?php echo $instance['controlnavthumbsreplace']; ?>" class="widefat" />
				</p>
				<p>
					<label><strong>keyboardNav:</strong></label><br />
					<input type="radio" id="<?php echo $this->get_field_id('keyboardnav'); ?>_default" name="<?php echo $this->get_field_name('keyboardnav'); ?>" value="" style="vertical-align: middle;"<?php if($instance['keyboardnav'] == '') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('keyboardnav'); ?>_default" style="vertical-align: middle;">default</label>
					<input type="radio" id="<?php echo $this->get_field_id('keyboardnav'); ?>_true" name="<?php echo $this->get_field_name('keyboardnav'); ?>" value="true" style="vertical-align: middle;"<?php if($instance['keyboardnav'] == 'true') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('keyboardnav'); ?>_true" style="vertical-align: middle;">true</label>
					<input type="radio" id="<?php echo $this->get_field_id('keyboardnav'); ?>_false" name="<?php echo $this->get_field_name('keyboardnav'); ?>" value="false" style="vertical-align: middle;"<?php if($instance['keyboardnav'] == 'false') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('keyboardnav'); ?>_false" style="vertical-align: middle;">false</label>
				</p>
				<p>
					<label><strong>pauseOnHover:</strong></label><br />
					<input type="radio" id="<?php echo $this->get_field_id('pauseonhover'); ?>_default" name="<?php echo $this->get_field_name('pauseonhover'); ?>" value="" style="vertical-align: middle;"<?php if($instance['pauseonhover'] == '') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('pauseonhover'); ?>_default" style="vertical-align: middle;">default</label>
					<input type="radio" id="<?php echo $this->get_field_id('pauseonhover'); ?>_true" name="<?php echo $this->get_field_name('pauseonhover'); ?>" value="true" style="vertical-align: middle;"<?php if($instance['pauseonhover'] == 'true') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('pauseonhover'); ?>_true" style="vertical-align: middle;">true</label>
					<input type="radio" id="<?php echo $this->get_field_id('pauseonhover'); ?>_false" name="<?php echo $this->get_field_name('pauseonhover'); ?>" value="false" style="vertical-align: middle;"<?php if($instance['pauseonhover'] == 'false') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('pauseonhover'); ?>_false" style="vertical-align: middle;">false</label>
				</p>
				<p>
					<label><strong>manualAdvance:</strong></label><br />
					<input type="radio" id="<?php echo $this->get_field_id('manualadvance'); ?>_default" name="<?php echo $this->get_field_name('manualadvance'); ?>" value="" style="vertical-align: middle;"<?php if($instance['manualadvance'] == '') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('manualadvance'); ?>_default" style="vertical-align: middle;">default</label>
					<input type="radio" id="<?php echo $this->get_field_id('manualadvance'); ?>_true" name="<?php echo $this->get_field_name('manualadvance'); ?>" value="true" style="vertical-align: middle;"<?php if($instance['manualadvance'] == 'true') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('manualadvance'); ?>_true" style="vertical-align: middle;">true</label>
					<input type="radio" id="<?php echo $this->get_field_id('manualadvance'); ?>_false" name="<?php echo $this->get_field_name('manualadvance'); ?>" value="false" style="vertical-align: middle;"<?php if($instance['manualadvance'] == 'false') { echo " checked=\"checked\""; } ?> /><label for="<?php echo $this->get_field_id('manualadvance'); ?>_false" style="vertical-align: middle;">false</label>
				</p>
				<p>
					<input type="checkbox" id="<?php echo $this->get_field_id('disablecaptions'); ?>" style="vertical-align: middle;" name="<?php echo $this->get_field_name('disablecaptions'); ?>" value="1"<?php if($instance['disablecaptions'] == '1') { echo " checked=\"checked\""; } ?> />
					<label for="<?php echo $this->get_field_id('disablecaptions'); ?>" style="vertical-align: middle;"><strong>Disable captions</strong></label><br />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('captionopacity'); ?>"><strong>captionOpacity:</strong></label>
					<input type="text" id="<?php echo $this->get_field_id('captionopacity'); ?>" name="<?php echo $this->get_field_name('captionopacity'); ?>" value="<?php echo $instance['captionopacity']; ?>" size="3" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('beforechange'); ?>"><strong>beforeChange:</strong></label><br />
					<input type="text" id="<?php echo $this->get_field_id('beforechange'); ?>" name="<?php echo $this->get_field_name('beforechange'); ?>" value="<?php echo $instance['beforechange']; ?>" class="widefat" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('afterchange'); ?>"><strong>afterChange:</strong></label><br />
					<input type="text" id="<?php echo $this->get_field_id('afterchange'); ?>" name="<?php echo $this->get_field_name('afterchange'); ?>" value="<?php echo $instance['afterchange']; ?>" class="widefat" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('slideshowend'); ?>"><strong>slideshowEnd:</strong></label><br />
					<input type="text" id="<?php echo $this->get_field_id('slideshowend'); ?>" name="<?php echo $this->get_field_name('slideshowend'); ?>" value="<?php echo $instance['slideshowend']; ?>" class="widefat" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('lastslide'); ?>"><strong>lastSlide:</strong></label><br />
					<input type="text" id="<?php echo $this->get_field_id('lastslide'); ?>" name="<?php echo $this->get_field_name('lastslide'); ?>" value="<?php echo $instance['lastslide']; ?>" class="widefat" />
				</p>
				<p>
					<label for="<?php echo $this->get_field_id('afterload'); ?>"><strong>afterLoad:</strong></label><br />
					<input type="text" id="<?php echo $this->get_field_id('afterload'); ?>" name="<?php echo $this->get_field_name('afterload'); ?>" value="<?php echo $instance['afterload']; ?>" class="widefat" />
				</p>
			</div>
		</div>
		<p><a href="#" onclick="jQuery(this).parent().prev('div.javascript_settings').toggle();return false;">Nivo Slider Settings</a></p>
		<?php
	}

	function update($new_instance, $old_instance) {
		$new_instance['title'] = esc_attr($new_instance['title']);
		return $new_instance;
	}
}